@@ -1,28 +1,30 @@
 import builtins
+import dataclasses
+import inspect
 import os
-from rich.repr import RichReprResult
+import re
 import sys
 from array import array
-from collections import Counter, defaultdict, deque, UserDict, UserList
-import dataclasses
+from collections import Counter, UserDict, UserList, defaultdict, deque
 from dataclasses import dataclass, fields, is_dataclass
 from inspect import isclass
 from itertools import islice
-import re
+from types import MappingProxyType
 from typing import (
-    DefaultDict,
     TYPE_CHECKING,
     Any,
     Callable,
+    DefaultDict,
     Dict,
     Iterable,
     List,
     Optional,
     Set,
-    Union,
     Tuple,
+    Union,
 )
-from types import MappingProxyType
+
+from rich.repr import RichReprResult
 
 try:
     import attr as _attr_module
@@ -30,7 +32,6 @@ except ImportError:  # pragma: no cover
     _attr_module = None  # type: ignore
 
 
-from .highlighter import ReprHighlighter
 from . import get_console
 from ._loop import loop_last
 from ._pick import pick_bool
@@ -51,9 +52,6 @@ if TYPE_CHECKING:
         RenderResult,
     )
 
-# Matches Jupyter's special methods
-_re_jupyter_repr = re.compile(f"^_repr_.+_$")
-
 
 def _is_attr_object(obj: Any) -> bool:
     """Check if an object was created with attrs module."""
@@ -78,10 +76,74 @@ def _is_dataclass_repr(obj: object) -> bool:
     # Catching all exceptions in case something is missing on a non CPython implementation
     try:
         return obj.__repr__.__code__.co_filename == dataclasses.__file__
-    except Exception:
+    except Exception:  # pragma: no coverage
         return False
 
 
+def _ipy_display_hook(
+    value: Any,
+    console: Optional["Console"] = None,
+    overflow: "OverflowMethod" = "ignore",
+    crop: bool = False,
+    indent_guides: bool = False,
+    max_length: Optional[int] = None,
+    max_string: Optional[int] = None,
+    expand_all: bool = False,
+) -> None:
+    from .console import ConsoleRenderable  # needed here to prevent circular import
+
+    # always skip rich generated jupyter renderables or None values
+    if isinstance(value, JupyterRenderable) or value is None:
+        return
+
+    console = console or get_console()
+    if console.is_jupyter:
+        # Delegate rendering to IPython if the object (and IPython) supports it
+        #  https://ipython.readthedocs.io/en/stable/config/integrating.html#rich-display
+        ipython_repr_methods = [
+            "_repr_html_",
+            "_repr_markdown_",
+            "_repr_json_",
+            "_repr_latex_",
+            "_repr_jpeg_",
+            "_repr_png_",
+            "_repr_svg_",
+            "_repr_mimebundle_",
+        ]
+        for repr_method in ipython_repr_methods:
+            method = getattr(value, repr_method, None)
+            if inspect.ismethod(method):
+                # Calling the method ourselves isn't ideal. The interface for the `_repr_*_` methods
+                #  specifies that if they return None, then they should not be rendered
+                #  by the notebook.
+                try:
+                    repr_result = method()
+                except Exception:
+                    continue  # If the method raises, treat it as if it doesn't exist, try any others
+                if repr_result is not None:
+                    return  # Delegate rendering to IPython
+
+    # certain renderables should start on a new line
+    if isinstance(value, ConsoleRenderable):
+        console.line()
+
+    console.print(
+        value
+        if isinstance(value, RichRenderable)
+        else Pretty(
+            value,
+            overflow=overflow,
+            indent_guides=indent_guides,
+            max_length=max_length,
+            max_string=max_string,
+            expand_all=expand_all,
+            margin=12,
+        ),
+        crop=crop,
+        new_line_start=True,
+    )
+
+
 def install(
     console: Optional["Console"] = None,
     overflow: "OverflowMethod" = "ignore",
@@ -106,8 +168,6 @@ def install(
     """
     from rich import get_console
 
-    from .console import ConsoleRenderable  # needed here to prevent circular import
-
     console = console or get_console()
     assert console is not None
 
@@ -131,37 +191,6 @@ def install(
             )
             builtins._ = value  # type: ignore
 
-    def ipy_display_hook(value: Any) -> None:  # pragma: no cover
-        assert console is not None
-        # always skip rich generated jupyter renderables or None values
-        if isinstance(value, JupyterRenderable) or value is None:
-            return
-        # on jupyter rich display, if using one of the special representations don't use rich
-        if console.is_jupyter and any(
-            _re_jupyter_repr.match(attr) for attr in dir(value)
-        ):
-            return
-
-        # certain renderables should start on a new line
-        if isinstance(value, ConsoleRenderable):
-            console.line()
-
-        console.print(
-            value
-            if isinstance(value, RichRenderable)
-            else Pretty(
-                value,
-                overflow=overflow,
-                indent_guides=indent_guides,
-                max_length=max_length,
-                max_string=max_string,
-                expand_all=expand_all,
-                margin=12,
-            ),
-            crop=crop,
-            new_line_start=True,
-        )
-
     try:  # pragma: no cover
         ip = get_ipython()  # type: ignore
         from IPython.core.formatters import BaseFormatter
@@ -171,7 +200,15 @@ def install(
 
             def __call__(self, value: Any) -> Any:
                 if self.pprint:
-                    return ipy_display_hook(value)
+                    return _ipy_display_hook(
+                        value,
+                        console=get_console(),
+                        overflow=overflow,
+                        indent_guides=indent_guides,
+                        max_length=max_length,
+                        max_string=max_string,
+                        expand_all=expand_all,
+                    )
                 else:
                     return repr(value)
 
@@ -196,6 +233,7 @@ class Pretty(JupyterMixin):
         max_length (int, optional): Maximum length of containers before abbreviating, or None for no abbreviation.
             Defaults to None.
         max_string (int, optional): Maximum length of string before truncating, or None to disable. Defaults to None.
+        max_depth (int, optional): Maximum depth of nested data structures, or None for no maximum. Defaults to None.
         expand_all (bool, optional): Expand all containers. Defaults to False.
         margin (int, optional): Subtrace a margin from width to force containers to expand earlier. Defaults to 0.
         insert_line (bool, optional): Insert a new line if the output has multiple new lines. Defaults to False.
@@ -213,6 +251,7 @@ class Pretty(JupyterMixin):
         indent_guides: bool = False,
         max_length: Optional[int] = None,
         max_string: Optional[int] = None,
+        max_depth: Optional[int] = None,
         expand_all: bool = False,
         margin: int = 0,
         insert_line: bool = False,
@@ -226,6 +265,7 @@ class Pretty(JupyterMixin):
         self.indent_guides = indent_guides
         self.max_length = max_length
         self.max_string = max_string
+        self.max_depth = max_depth
         self.expand_all = expand_all
         self.margin = margin
         self.insert_line = insert_line
@@ -239,6 +279,7 @@ class Pretty(JupyterMixin):
             indent_size=self.indent_size,
             max_length=self.max_length,
             max_string=self.max_string,
+            max_depth=self.max_depth,
             expand_all=self.expand_all,
         )
         pretty_text = Text(
@@ -474,7 +515,10 @@ class _Line:
 
 
 def traverse(
-    _object: Any, max_length: Optional[int] = None, max_string: Optional[int] = None
+    _object: Any,
+    max_length: Optional[int] = None,
+    max_string: Optional[int] = None,
+    max_depth: Optional[int] = None,
 ) -> Node:
     """Traverse object and generate a tree.
 
@@ -484,6 +528,8 @@ def traverse(
             Defaults to None.
         max_string (int, optional): Maximum length of string before truncating, or None to disable truncating.
             Defaults to None.
+        max_depth (int, optional): Maximum depth of data structures, or None for no maximum.
+            Defaults to None.
 
     Returns:
         Node: The root of a tree structure which can be used to render a pretty repr.
@@ -509,11 +555,13 @@ def traverse(
     push_visited = visited_ids.add
     pop_visited = visited_ids.remove
 
-    def _traverse(obj: Any, root: bool = False) -> Node:
+    def _traverse(obj: Any, root: bool = False, depth: int = 0) -> Node:
         """Walk the object depth first."""
+
         obj_type = type(obj)
         py_version = (sys.version_info.major, sys.version_info.minor)
         children: List[Node]
+        reached_max_depth = max_depth is not None and depth >= max_depth
 
         def iter_rich_args(rich_args: Any) -> Iterable[Union[Any, Tuple[str, Any]]]:
             for arg in rich_args:
@@ -554,33 +602,37 @@ def traverse(
             if args:
                 children = []
                 append = children.append
-                if angular:
-                    node = Node(
-                        open_brace=f"<{class_name} ",
-                        close_brace=">",
-                        children=children,
-                        last=root,
-                        separator=" ",
-                    )
+
+                if reached_max_depth:
+                    node = Node(value_repr=f"...")
                 else:
-                    node = Node(
-                        open_brace=f"{class_name}(",
-                        close_brace=")",
-                        children=children,
-                        last=root,
-                    )
-                for last, arg in loop_last(args):
-                    if isinstance(arg, tuple):
-                        key, child = arg
-                        child_node = _traverse(child)
-                        child_node.last = last
-                        child_node.key_repr = key
-                        child_node.key_separator = "="
-                        append(child_node)
+                    if angular:
+                        node = Node(
+                            open_brace=f"<{class_name} ",
+                            close_brace=">",
+                            children=children,
+                            last=root,
+                            separator=" ",
+                        )
                     else:
-                        child_node = _traverse(arg)
-                        child_node.last = last
-                        append(child_node)
+                        node = Node(
+                            open_brace=f"{class_name}(",
+                            close_brace=")",
+                            children=children,
+                            last=root,
+                        )
+                    for last, arg in loop_last(args):
+                        if isinstance(arg, tuple):
+                            key, child = arg
+                            child_node = _traverse(child, depth=depth + 1)
+                            child_node.last = last
+                            child_node.key_repr = key
+                            child_node.key_separator = "="
+                            append(child_node)
+                        else:
+                            child_node = _traverse(arg, depth=depth + 1)
+                            child_node.last = last
+                            append(child_node)
             else:
                 node = Node(
                     value_repr=f"<{class_name}>" if angular else f"{class_name}()",
@@ -593,40 +645,43 @@ def traverse(
 
             attr_fields = _get_attr_fields(obj)
             if attr_fields:
-                node = Node(
-                    open_brace=f"{obj.__class__.__name__}(",
-                    close_brace=")",
-                    children=children,
-                    last=root,
-                )
+                if reached_max_depth:
+                    node = Node(value_repr=f"...")
+                else:
+                    node = Node(
+                        open_brace=f"{obj.__class__.__name__}(",
+                        close_brace=")",
+                        children=children,
+                        last=root,
+                    )
 
-                def iter_attrs() -> Iterable[
-                    Tuple[str, Any, Optional[Callable[[Any], str]]]
-                ]:
-                    """Iterate over attr fields and values."""
-                    for attr in attr_fields:
-                        if attr.repr:
-                            try:
-                                value = getattr(obj, attr.name)
-                            except Exception as error:
-                                # Can happen, albeit rarely
-                                yield (attr.name, error, None)
-                            else:
-                                yield (
-                                    attr.name,
-                                    value,
-                                    attr.repr if callable(attr.repr) else None,
-                                )
+                    def iter_attrs() -> Iterable[
+                        Tuple[str, Any, Optional[Callable[[Any], str]]]
+                    ]:
+                        """Iterate over attr fields and values."""
+                        for attr in attr_fields:
+                            if attr.repr:
+                                try:
+                                    value = getattr(obj, attr.name)
+                                except Exception as error:
+                                    # Can happen, albeit rarely
+                                    yield (attr.name, error, None)
+                                else:
+                                    yield (
+                                        attr.name,
+                                        value,
+                                        attr.repr if callable(attr.repr) else None,
+                                    )
 
-                for last, (name, value, repr_callable) in loop_last(iter_attrs()):
-                    if repr_callable:
-                        child_node = Node(value_repr=str(repr_callable(value)))
-                    else:
-                        child_node = _traverse(value)
-                    child_node.last = last
-                    child_node.key_repr = name
-                    child_node.key_separator = "="
-                    append(child_node)
+                    for last, (name, value, repr_callable) in loop_last(iter_attrs()):
+                        if repr_callable:
+                            child_node = Node(value_repr=str(repr_callable(value)))
+                        else:
+                            child_node = _traverse(value, depth=depth + 1)
+                        child_node.last = last
+                        child_node.key_repr = name
+                        child_node.key_separator = "="
+                        append(child_node)
             else:
                 node = Node(
                     value_repr=f"{obj.__class__.__name__}()", children=[], last=root
@@ -646,21 +701,26 @@ def traverse(
 
             children = []
             append = children.append
-            node = Node(
-                open_brace=f"{obj.__class__.__name__}(",
-                close_brace=")",
-                children=children,
-                last=root,
-            )
+            if reached_max_depth:
+                node = Node(value_repr=f"...")
+            else:
+                node = Node(
+                    open_brace=f"{obj.__class__.__name__}(",
+                    close_brace=")",
+                    children=children,
+                    last=root,
+                )
 
-            for last, field in loop_last(field for field in fields(obj) if field.repr):
-                child_node = _traverse(getattr(obj, field.name))
-                child_node.key_repr = field.name
-                child_node.last = last
-                child_node.key_separator = "="
-                append(child_node)
+                for last, field in loop_last(
+                    field for field in fields(obj) if field.repr
+                ):
+                    child_node = _traverse(getattr(obj, field.name), depth=depth + 1)
+                    child_node.key_repr = field.name
+                    child_node.last = last
+                    child_node.key_separator = "="
+                    append(child_node)
 
-            pop_visited(obj_id)
+                pop_visited(obj_id)
 
         elif isinstance(obj, _CONTAINERS):
             for container_type in _CONTAINERS:
@@ -676,7 +736,9 @@ def traverse(
 
             open_brace, close_brace, empty = _BRACES[obj_type](obj)
 
-            if obj_type.__repr__ != type(obj).__repr__:
+            if reached_max_depth:
+                node = Node(value_repr=f"...", last=root)
+            elif obj_type.__repr__ != type(obj).__repr__:
                 node = Node(value_repr=to_repr(obj), last=root)
             elif obj:
                 children = []
@@ -695,7 +757,7 @@ def traverse(
                     if max_length is not None:
                         iter_items = islice(iter_items, max_length)
                     for index, (key, child) in enumerate(iter_items):
-                        child_node = _traverse(child)
+                        child_node = _traverse(child, depth=depth + 1)
                         child_node.key_repr = to_repr(key)
                         child_node.last = index == last_item_index
                         append(child_node)
@@ -704,7 +766,7 @@ def traverse(
                     if max_length is not None:
                         iter_values = islice(iter_values, max_length)
                     for index, child in enumerate(iter_values):
-                        child_node = _traverse(child)
+                        child_node = _traverse(child, depth=depth + 1)
                         child_node.last = index == last_item_index
                         append(child_node)
                 if max_length is not None and num_items > max_length:
@@ -729,6 +791,7 @@ def pretty_repr(
     indent_size: int = 4,
     max_length: Optional[int] = None,
     max_string: Optional[int] = None,
+    max_depth: Optional[int] = None,
     expand_all: bool = False,
 ) -> str:
     """Prettify repr string by expanding on to new lines to fit within a given width.
@@ -741,6 +804,8 @@ def pretty_repr(
             Defaults to None.
         max_string (int, optional): Maximum length of string before truncating, or None to disable truncating.
             Defaults to None.
+        max_depth (int, optional): Maximum depth of nested data structure, or None for no depth.
+            Defaults to None.
         expand_all (bool, optional): Expand all containers regardless of available width. Defaults to False.
 
     Returns:
@@ -750,7 +815,9 @@ def pretty_repr(
     if isinstance(_object, Node):
         node = _object
     else:
-        node = traverse(_object, max_length=max_length, max_string=max_string)
+        node = traverse(
+            _object, max_length=max_length, max_string=max_string, max_depth=max_depth
+        )
     repr_str = node.render(
         max_width=max_width, indent_size=indent_size, expand_all=expand_all
     )
@@ -764,6 +831,7 @@ def pprint(
     indent_guides: bool = True,
     max_length: Optional[int] = None,
     max_string: Optional[int] = None,
+    max_depth: Optional[int] = None,
     expand_all: bool = False,
 ) -> None:
     """A convenience function for pretty printing.
@@ -774,6 +842,7 @@ def pprint(
         max_length (int, optional): Maximum length of containers before abbreviating, or None for no abbreviation.
             Defaults to None.
         max_string (int, optional): Maximum length of strings before truncating, or None to disable. Defaults to None.
+        max_depth (int, optional): Maximum depth for nested data structures, or None for unlimited depth. Defaults to None.
         indent_guides (bool, optional): Enable indentation guides. Defaults to True.
         expand_all (bool, optional): Expand all containers. Defaults to False.
     """
@@ -783,6 +852,7 @@ def pprint(
             _object,
             max_length=max_length,
             max_string=max_string,
+            max_depth=max_depth,
             indent_guides=indent_guides,
             expand_all=expand_all,
             overflow="ignore",
